# Copyright (c) 2017, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Standard files documentation fragment
    DOCUMENTATION = r"""
description:
  - This module allows one to (re)generate OpenSSL certificate signing requests.
  - This module supports the subjectAltName, keyUsage, extendedKeyUsage, basicConstraints and OCSP Must Staple extensions.
attributes:
  diff_mode:
    support: full
  idempotent:
    support: full
requirements:
  - cryptography >= 3.3
options:
  digest:
    description:
      - The digest used when signing the certificate signing request with the private key.
    type: str
    default: sha256
  privatekey_path:
    description:
      - The path to the private key to use when signing the certificate signing request.
      - Either O(privatekey_path) or O(privatekey_content) must be specified if O(state) is V(present), but not both.
    type: path
  privatekey_content:
    description:
      - The content of the private key to use when signing the certificate signing request.
      - Either O(privatekey_path) or O(privatekey_content) must be specified if O(state) is V(present), but not both.
    type: str
  privatekey_passphrase:
    description:
      - The passphrase for the private key.
      - This is required if the private key is password protected.
    type: str
  version:
    description:
      - The version of the certificate signing request.
      - The only allowed value according to L(RFC 2986,https://tools.ietf.org/html/rfc2986#section-4.1) is 1.
      - This option no longer accepts unsupported values since community.crypto 2.0.0.
    type: int
    default: 1
    choices:
      - 1
  subject:
    description:
      - Key/value pairs that will be present in the subject name field of the certificate signing request.
      - If you need to specify more than one value with the same key, use a list as value.
      - If the order of the components is important, use O(subject_ordered).
      - Mutually exclusive with O(subject_ordered).
    type: dict
  subject_ordered:
    description:
      - A list of dictionaries, where every dictionary must contain one key/value pair. This key/value pair will be present
        in the subject name field of the certificate signing request.
      - If you want to specify more than one value with the same key in a row, you can use a list as value.
      - Mutually exclusive with O(subject), and any other subject field option, such as O(country_name), O(state_or_province_name),
        O(locality_name), O(organization_name), O(organizational_unit_name), O(common_name), or O(email_address).
    type: list
    elements: dict
    version_added: 2.0.0
  country_name:
    description:
      - The countryName field of the certificate signing request subject.
    type: str
    aliases:
      - C
      - countryName
  state_or_province_name:
    description:
      - The stateOrProvinceName field of the certificate signing request subject.
    type: str
    aliases:
      - ST
      - stateOrProvinceName
  locality_name:
    description:
      - The localityName field of the certificate signing request subject.
    type: str
    aliases:
      - L
      - localityName
  organization_name:
    description:
      - The organizationName field of the certificate signing request subject.
    type: str
    aliases:
      - O
      - organizationName
  organizational_unit_name:
    description:
      - The organizationalUnitName field of the certificate signing request subject.
    type: str
    aliases:
      - OU
      - organizationalUnitName
  common_name:
    description:
      - The commonName field of the certificate signing request subject.
    type: str
    aliases:
      - CN
      - commonName
  email_address:
    description:
      - The emailAddress field of the certificate signing request subject.
    type: str
    aliases:
      - E
      - emailAddress
  subject_alt_name:
    description:
      - Subject Alternative Name (SAN) extension to attach to the certificate signing request.
      - Values must be prefixed by their options. (These are C(email), C(URI), C(DNS), C(RID), C(IP), C(dirName), C(otherName),
        and the ones specific to your CA).
      - Note that if no SAN is specified, but a common name, the common name will be added as a SAN except if O(use_common_name_for_san)
        is set to V(false).
      - More at U(https://tools.ietf.org/html/rfc5280#section-4.2.1.6).
    type: list
    elements: str
    aliases:
      - subjectAltName
  subject_alt_name_critical:
    description:
      - Should the subjectAltName extension be considered as critical.
    type: bool
    default: false
    aliases:
      - subjectAltName_critical
  use_common_name_for_san:
    description:
      - If set to V(true), the module will fill the common name in for O(subject_alt_name) with C(DNS:) prefix if no SAN is
        specified.
    type: bool
    default: true
    aliases:
      - useCommonNameForSAN
  key_usage:
    description:
      - This defines the purpose (for example encipherment, signature, certificate signing) of the key contained in the certificate.
    type: list
    elements: str
    aliases:
      - keyUsage
  key_usage_critical:
    description:
      - Should the keyUsage extension be considered as critical.
    type: bool
    default: false
    aliases:
      - keyUsage_critical
  extended_key_usage:
    description:
      - Additional restrictions (for example client authentication, server authentication) on the allowed purposes for which
        the public key may be used.
    type: list
    elements: str
    aliases:
      - extKeyUsage
      - extendedKeyUsage
  extended_key_usage_critical:
    description:
      - Should the extkeyUsage extension be considered as critical.
    type: bool
    default: false
    aliases:
      - extKeyUsage_critical
      - extendedKeyUsage_critical
  basic_constraints:
    description:
      - Indicates basic constraints, such as if the certificate is a CA.
    type: list
    elements: str
    aliases:
      - basicConstraints
  basic_constraints_critical:
    description:
      - Should the basicConstraints extension be considered as critical.
    type: bool
    default: false
    aliases:
      - basicConstraints_critical
  ocsp_must_staple:
    description:
      - Indicates that the certificate should contain the OCSP Must Staple extension (U(https://tools.ietf.org/html/rfc7633)).
    type: bool
    default: false
    aliases:
      - ocspMustStaple
  ocsp_must_staple_critical:
    description:
      - Should the OCSP Must Staple extension be considered as critical.
      - Note that according to the RFC, this extension should not be marked as critical, as old clients not knowing about
        OCSP Must Staple are required to reject such certificates (see U(https://tools.ietf.org/html/rfc7633#section-4)).
    type: bool
    default: false
    aliases:
      - ocspMustStaple_critical
  name_constraints_permitted:
    description:
      - For CA certificates, this specifies a list of identifiers which describe subtrees of names that this CA is allowed
        to issue certificates for.
      - Values must be prefixed by their options. (That is, C(email), C(URI), C(DNS), C(RID), C(IP), C(dirName), C(otherName),
        and the ones specific to your CA).
    type: list
    elements: str
  name_constraints_excluded:
    description:
      - For CA certificates, this specifies a list of identifiers which describe subtrees of names that this CA is B(not)
        allowed to issue certificates for.
      - Values must be prefixed by their options. (That is, C(email), C(URI), C(DNS), C(RID), C(IP), C(dirName), C(otherName),
        and the ones specific to your CA).
    type: list
    elements: str
  name_constraints_critical:
    description:
      - Should the Name Constraints extension be considered as critical.
    type: bool
    default: false
  select_crypto_backend:
    description:
      - Determines which crypto backend to use.
      - The default choice is V(auto), which tries to use C(cryptography) if available.
      - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
      - Note that with community.crypto 3.0.0, all values behave the same.
        This option will be deprecated in a later version.
        We recommend to not set it explicitly.
    type: str
    default: auto
    choices: [auto, cryptography]
  create_subject_key_identifier:
    description:
      - Create the Subject Key Identifier from the public key.
      - Please note that commercial CAs can ignore the value, respectively use a value of their own choice instead. Specifying
        this option is mostly useful for self-signed certificates or for own CAs.
    type: bool
    default: false
  subject_key_identifier:
    description:
      - The subject key identifier as a hex string, where two bytes are separated by colons.
      - 'Example: V(00:11:22:33:44:55:66:77:88:99:aa:bb:cc:dd:ee:ff:00:11:22:33).'
      - Please note that commercial CAs ignore this value, respectively use a value of their own choice. Specifying this option
        is mostly useful for self-signed certificates or for own CAs.
      - Note that this option can only be used if O(create_subject_key_identifier) is V(false).
    type: str
  authority_key_identifier:
    description:
      - The authority key identifier as a hex string, where two bytes are separated by colons.
      - 'Example: V(00:11:22:33:44:55:66:77:88:99:aa:bb:cc:dd:ee:ff:00:11:22:33).'
      - Please note that commercial CAs ignore this value, respectively use a value of their own choice. Specifying this option
        is mostly useful for self-signed certificates or for own CAs.
      - The C(AuthorityKeyIdentifier) extension will only be added if at least one of O(authority_key_identifier), O(authority_cert_issuer)
        and O(authority_cert_serial_number) is specified.
    type: str
  authority_cert_issuer:
    description:
      - Names that will be present in the authority cert issuer field of the certificate signing request.
      - Values must be prefixed by their options. (That is, C(email), C(URI), C(DNS), C(RID), C(IP), C(dirName), C(otherName),
        and the ones specific to your CA).
      - 'Example: V(DNS:ca.example.org).'
      - If specified, O(authority_cert_serial_number) must also be specified.
      - Please note that commercial CAs ignore this value, respectively use a value of their own choice. Specifying this option
        is mostly useful for self-signed certificates or for own CAs.
      - The C(AuthorityKeyIdentifier) extension will only be added if at least one of O(authority_key_identifier), O(authority_cert_issuer)
        and O(authority_cert_serial_number) is specified.
    type: list
    elements: str
  authority_cert_serial_number:
    description:
      - The authority cert serial number.
      - If specified, O(authority_cert_issuer) must also be specified.
      - Please note that commercial CAs ignore this value, respectively use a value of their own choice. Specifying this option
        is mostly useful for self-signed certificates or for own CAs.
      - The C(AuthorityKeyIdentifier) extension will only be added if at least one of O(authority_key_identifier), O(authority_cert_issuer)
        and O(authority_cert_serial_number) is specified.
      - This option accepts an B(integer). If you want to provide serial numbers as colon-separated hex strings, such as C(11:22:33),
        you need to convert them to an integer with P(community.crypto.parse_serial#filter).
    type: int
  crl_distribution_points:
    description:
      - Allows to specify one or multiple CRL distribution points.
    type: list
    elements: dict
    suboptions:
      full_name:
        description:
          - Describes how the CRL can be retrieved.
          - Mutually exclusive with O(crl_distribution_points[].relative_name).
          - 'Example: V(URI:https://ca.example.com/revocations.crl).'
        type: list
        elements: str
      relative_name:
        description:
          - Describes how the CRL can be retrieved relative to the CRL issuer.
          - Mutually exclusive with O(crl_distribution_points[].full_name).
          - 'Example: V(/CN=example.com).'
        type: list
        elements: str
      crl_issuer:
        description:
          - Information about the issuer of the CRL.
        type: list
        elements: str
      reasons:
        description:
          - List of reasons that this distribution point can be used for when performing revocation checks.
        type: list
        elements: str
        choices:
          - key_compromise
          - ca_compromise
          - affiliation_changed
          - superseded
          - cessation_of_operation
          - certificate_hold
          - privilege_withdrawn
          - aa_compromise
    version_added: 1.4.0
notes:
  - If the certificate signing request already exists it will be checked whether subjectAltName, keyUsage, extendedKeyUsage
    and basicConstraints only contain the requested values, whether OCSP Must Staple is as requested, and if the request was
    signed by the given private key.
seealso:
  - module: community.crypto.x509_certificate
  - module: community.crypto.x509_certificate_pipe
  - module: community.crypto.openssl_dhparam
  - module: community.crypto.openssl_pkcs12
  - module: community.crypto.openssl_privatekey
  - module: community.crypto.openssl_privatekey_pipe
  - module: community.crypto.openssl_publickey
  - module: community.crypto.openssl_csr_info
  - plugin: community.crypto.parse_serial
    plugin_type: filter
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_switch_routing_interfaces_dhcp
short_description: Resource module for devices _switch _routing _interfaces _dhcp
description:
  - Manage operation update of the resource devices _switch _routing _interfaces _dhcp.
  - Update a layer 3 interface DHCP configuration for a switch.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  bootFileName:
    description: The PXE boot server filename for the DHCP server running on the switch
      interface.
    type: str
  bootNextServer:
    description: The PXE boot server IP for the DHCP server running on the switch
      interface.
    type: str
  bootOptionsEnabled:
    description: Enable DHCP boot options to provide PXE boot options configs for
      the dhcp server running on the switch interface.
    type: bool
  dhcpLeaseTime:
    description: The DHCP lease time config for the dhcp server running on switch
      interface ('30 minutes', '1 hour', '4 hours', '12 hours', '1 day' or '1 week').
    type: str
  dhcpMode:
    description: The DHCP mode options for the switch interface ('dhcpDisabled', 'dhcpRelay'
      or 'dhcpServer').
    type: str
  dhcpOptions:
    description: Array of DHCP options consisting of code, type and value for the
      DHCP server running on the switch interface.
    elements: dict
    suboptions:
      code:
        description: The code for DHCP option which should be from 2 to 254.
        type: str
      type:
        description: The type of the DHCP option which should be one of ('text', 'ip',
          'integer' or 'hex').
        type: str
      value:
        description: The value of the DHCP option.
        type: str
    type: list
  dhcpRelayServerIps:
    description: The DHCP relay server IPs to which DHCP packets would get relayed
      for the switch interface.
    elements: str
    type: list
  dnsCustomNameservers:
    description: The DHCP name server IPs when DHCP name server option is 'custom'.
    elements: str
    type: list
  dnsNameserversOption:
    description: The DHCP name server option for the dhcp server running on the switch
      interface ('googlePublicDns', 'openDns' or 'custom').
    type: str
  fixedIpAssignments:
    description: Array of DHCP fixed IP assignments for the DHCP server running on
      the switch interface.
    elements: dict
    suboptions:
      ip:
        description: The IP address of the client which has fixed IP address assigned
          to it.
        type: str
      mac:
        description: The MAC address of the client which has fixed IP address.
        type: str
      name:
        description: The name of the client which has fixed IP address.
        type: str
    type: list
  interfaceId:
    description: InterfaceId path parameter. Interface ID.
    type: str
  reservedIpRanges:
    description: Array of DHCP reserved IP assignments for the DHCP server running
      on the switch interface.
    elements: dict
    suboptions:
      comment:
        description: The comment for the reserved IP range.
        type: str
      end:
        description: The ending IP address of the reserved IP range.
        type: str
      start:
        description: The starting IP address of the reserved IP range.
        type: str
    type: list
  serial:
    description: Serial path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch updateDeviceSwitchRoutingInterfaceDhcp
    description: Complete reference of the updateDeviceSwitchRoutingInterfaceDhcp
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-device-switch-routing-interface-dhcp
notes:
  - SDK Method used are
    switch.Switch.update_device_switch_routing_interface_dhcp,
  - Paths used are
    put /devices/{serial}/switch/routing/interfaces/{interfaceId}/dhcp,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.devices_switch_routing_interfaces_dhcp:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    bootFileName: home_boot_file
    bootNextServer: 1.2.3.4
    bootOptionsEnabled: true
    dhcpLeaseTime: 1 day
    dhcpMode: dhcpServer
    dhcpOptions:
      - code: '5'
        type: text
        value: five
    dhcpRelayServerIps:
      - 1.2.3.4
    dnsCustomNameservers:
      - 8.8.8.8, 8.8.4.4
    dnsNameserversOption: custom
    fixedIpAssignments:
      - ip: 192.168.1.12
        mac: 22:33:44:55:66:77
        name: Cisco Meraki valued client
    interfaceId: string
    reservedIpRanges:
      - comment: A reserved IP range
        end: 192.168.1.10
        start: 192.168.1.1
    serial: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "dhcpMode": "string",
      "dhcpRelayServerIps": [
        "string"
      ],
      "dhcpLeaseTime": "string",
      "dnsNameserversOption": "string",
      "dnsCustomNameservers": [
        "string"
      ],
      "bootOptionsEnabled": true,
      "bootNextServer": "string",
      "bootFileName": "string",
      "dhcpOptions": [
        {
          "code": "string",
          "type": "string",
          "value": "string"
        }
      ],
      "reservedIpRanges": [
        {
          "start": "string",
          "end": "string",
          "comment": "string"
        }
      ],
      "fixedIpAssignments": [
        {
          "name": "string",
          "mac": "string",
          "ip": "string"
        }
      ]
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios_bfd_templates config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import (
    Facts,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.bfd_templates import (
    Bfd_templatesTemplate,
)


class Bfd_templates(ResourceModule):
    """
    The ios_bfd_templates config class
    """

    def __init__(self, module):
        super(Bfd_templates, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="bfd_templates",
            tmplt=Bfd_templatesTemplate(),
        )
        self.parsers = ["interval", "dampening", "authentication", "echo"]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = {entry["name"]: entry for entry in self.want}
        haved = {entry["name"]: entry for entry in self.have}

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        if self.state == "deleted":
            # Filter to only templates specified in want, or all if want is empty
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            # For deleted state, compare empty want with have to generate  no commands
            for k, have in haved.items():
                self._compare(want={"name": have["name"], "hop": have["hop"]}, have=have)

        # if state is purged, remove entire templates at top level
        elif self.state == "purged":
            # Filter to only templates specified in want, or all if want is empty
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            for k, have in haved.items():
                self.purge(have)

        # remove superfluous config for overridden
        elif self.state == "overridden":
            for k, have in haved.items():
                if k not in wantd:
                    self.purge(have)
            # Process templates in the original order from self.want to ensure deterministic output
            for entry in self.want:
                k = entry["name"]
                if k in wantd:
                    self._compare(want=wantd[k], have=haved.pop(k, {}))

        else:
            # Process templates in the original order from self.want to ensure deterministic output
            for entry in self.want:
                k = entry["name"]
                if k in wantd:
                    self._compare(want=wantd[k], have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Bfd_templates network resource.
        """
        begin = len(self.commands)

        # Handle echo removal - needs to come first in command order
        # Check if echo should be removed (present in have but not in want, or explicitly False in want)
        parsers = self.parsers
        if have.get("echo") and not want.get("echo"):
            self.commands.append("no echo")
            # Exclude echo from parsers since we've already handled it
            parsers = [p for p in self.parsers if p != "echo"]

        self.compare(parsers=parsers, want=want, have=have)

        # Handle authentication replacement - need to negate old auth before adding new one
        if have.get("authentication") and want.get("authentication"):
            if have["authentication"] != want["authentication"]:
                # Find the index of the new authentication command
                auth_cmd_prefix = (
                    f"authentication {want['authentication']['type'].replace('_', '-')}"
                )
                for i in range(begin, len(self.commands)):
                    if self.commands[i].startswith(auth_cmd_prefix):
                        # Insert the no command before the new authentication
                        no_auth_cmd = self._tmplt.render(have, "authentication", True)
                        self.commands.insert(i, no_auth_cmd)
                        break

        # Insert the bfd-template header command at the beginning
        if len(self.commands) != begin:
            self.commands.insert(begin, self._tmplt.render(want or have, "name", False))

    def purge(self, have):
        """Handle operation for purged state - removes entire template"""
        self.commands.append(self._tmplt.render(have, "name", True))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_l3out_interface_routing_policy
short_description: Manage L3Out Interface Routing Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage L3Out Interface Routing Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Sabari Jaganathan (@sajagana)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Tenant Policy Template.
    type: str
    required: true
  name:
    description:
    - The name of the L3Out Interface Routing Policy.
    type: str
    aliases: [ l3out_interface_routing_policy_name ]
  uuid:
    description:
    - The UUID of the L3Out Interface Routing Policy.
    - This parameter is required when the O(name) needs to be updated.
    type: str
    aliases: [ l3out_interface_routing_policy_uuid ]
  description:
    description:
    - The description of the L3Out Interface Routing Policy.
    type: str
  bfd_multi_hop_settings:
    description:
    - The Bidirectional Forwarding Detection (BFD) multi-hop configuration of the L3Out Interface Routing Policy.
    type: dict
    suboptions:
      state:
        description:
        - Use C(enabled) to configure the BFD MultiHop Settings.
        - Use C(disabled) to remove the BFD MultiHop Settings.
        type: str
        choices: [ enabled, disabled ]
      admin_state:
        description:
        - The administrative state of the BFD MultiHop Settings.
        - Defaults to C(enabled) when unset during creation.
        type: str
        choices: [ enabled, disabled ]
      detection_multiplier:
        description:
        - The detection multiplier of the BFD MultiHop Settings.
        - Defaults to 3 when unset during creation.
        - The value must be between 1 and 50.
        type: int
      min_receive_interval:
        description:
        - The minimum receive interval of the BFD MultiHop Settings.
        - Defaults to 250 when unset during creation.
        - The value must be between 250 and 999 microseconds.
        type: int
      min_transmit_interval:
        description:
        - The minimum transmit interval of the BFD MultiHop Settings.
        - Defaults to 250 when unset during creation.
        - The value must be between 250 and 999 microseconds.
        type: int
  bfd_settings:
    description:
    - The BFD Settings of the L3Out Interface Routing Policy.
    type: dict
    suboptions:
      state:
        description:
        - Use C(enabled) to configure the BFD Settings.
        - Use C(disabled) to remove the BFD Settings.
        type: str
        choices: [ enabled, disabled ]
      admin_state:
        description:
        - The administrative state of the BFD Settings.
        - Defaults to C(enabled) when unset during creation.
        type: str
        choices: [ enabled, disabled ]
      detection_multiplier:
        description:
        - The detection multiplier of the BFD Settings.
        - Defaults to 3 when unset during creation.
        - The value must be between 1 and 50.
        type: int
      min_receive_interval:
        description:
        - The minimum receive interval in microseconds of the BFD Settings.
        - Defaults to 50 when unset during creation.
        - The value must be between 50 and 999.
        type: int
      min_transmit_interval:
        description:
        - The minimum transmit interval in microseconds of the BFD Settings.
        - Defaults to 50 when unset during creation.
        - The value must be between 50 and 999.
        type: int
      echo_receive_interval:
        description:
        - The echo receive interval in microseconds of the BFD Settings
        - Defaults to 50 when unset during creation.
        - The value must be between 50 and 999.
        type: int
      echo_admin_state:
        description:
        - The echo administrative state of the BFD Settings.
        - Defaults to C(enabled) when unset during creation.
        type: str
        choices: [ enabled, disabled ]
      interface_control:
        description:
        - The interface control of the BFD Settings.
        - Defaults to C(disabled) when unset during creation.
        type: str
        choices: [ enabled, disabled ]
  ospf_interface_settings:
    description:
    - The Open Shortest Path First (OSPF) Interface Settings of the L3Out Interface Routing Policy.
    type: dict
    suboptions:
      state:
        description:
        - Use C(enabled) to configure the OSPF Interface Settings.
        - Use C(disabled) to remove the OSPF Interface Settings.
        type: str
        choices: [ enabled, disabled ]
      network_type:
        description:
        - The network type of the OSPF Interface Settings.
        - Defaults to C(broadcast) when unset during creation.
        type: str
        choices: [ broadcast, point_to_point ]
      priority:
        description:
        - The priority of the OSPF Interface Settings.
        - Defaults to 1 when unset during creation.
        - The value must be between 0 and 255.
        type: int
      cost_of_interface:
        description:
        - The cost of the OSPF Interface Settings.
        - Defaults to 0 when unset during creation.
        - The value must be between 0 and 65535.
        type: int
      hello_interval:
        description:
        - The hello interval in seconds of the OSPF Interface Settings.
        - Defaults to 10 when unset during creation.
        - The value must be between 1 and 65535.
        type: int
      dead_interval:
        description:
        - The dead interval in seconds of the OSPF Interface Settings.
        - Defaults to 40 when unset during creation.
        - The value must be between 1 and 65535.
        type: int
      retransmit_interval:
        description:
        - The retransmit interval in seconds of the OSPF Interface Settings.
        - Defaults to 5 when unset during creation.
        - The value must be between 1 and 65535.
        type: int
      transmit_delay:
        description:
        - The transmit delay in seconds of the OSPF Interface Settings.
        - Defaults to 1 when unset during creation.
        - The value must be between 1 and 450.
        type: int
      advertise_subnet:
        description:
        - The advertise subnet of the OSPF Interface Settings.
        - Defaults to C(disabled) when unset during creation.
        type: str
        choices: [ enabled, disabled ]
      bfd:
        description:
        - Enables or disables BFD in the OSPF Interface Settings.
        - Defaults to C(disabled) when unset during creation.
        type: str
        choices: [ enabled, disabled ]
      mtu_ignore:
        description:
        - Enables or disables ignoring the Maximum Transmission Unit (MTU) in the OSPF Interface Settings.
        - Defaults to C(disabled) when unset during creation.
        type: str
        choices: [ enabled, disabled ]
      passive_participation:
        description:
        - Enables or disables passive participation in the OSPF Interface Settings.
        - Defaults to C(disabled) when unset during creation.
        type: str
        choices: [ enabled, disabled ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant Policy Template.
seealso:
- module: cisco.mso.ndo_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new L3Out Interface Routing Policy with default values
  cisco.mso.ndo_l3out_interface_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: irp_1
    bfd_settings:
      state: enabled
    bfd_multi_hop_settings:
      state: enabled
    state: present
  register: irp_1_present

- name: Update an existing L3Out Interface Routing Policy with UUID
  cisco.mso.ndo_l3out_interface_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ irp_1_present.current.uuid }}"
    name: irp_1_updated
    bfd_multi_hop_settings:
      admin_state: disabled
      detection_multiplier: 10
      min_receive_interval: 255
      min_transmit_interval: 255
    bfd_settings:
      admin_state: disabled
      detection_multiplier: 10
      min_receive_interval: 266
      min_transmit_interval: 266
      echo_receive_interval: 60
      echo_admin_state: disabled
      interface_control: enabled
    ospf_interface_settings:
      network_type: point_to_point
      priority: 10
      cost_of_interface: 100
      advertise_subnet: enabled
      bfd: enabled
      mtu_ignore: enabled
      passive_participation: enabled
      hello_interval: 20
      dead_interval: 30
      retransmit_interval: 20
      transmit_delay: 10
    state: present
  register: irp_1_present

- name: Clear an existing L3Out Interface Routing Policy BFD and OSPF interface settings
  cisco.mso.ndo_l3out_interface_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ irp_1_present.current.uuid }}"
    bfd_settings:
      state: disabled
    ospf_interface_settings:
      state: disabled
    state: present

- name: Query an L3Out Interface Routing Policy with name
  cisco.mso.ndo_l3out_interface_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: irp_1_updated
    state: query
  register: query_with_name

- name: Query an L3Out Interface Routing Policy with UUID
  cisco.mso.ndo_l3out_interface_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ irp_1_present.current.uuid }}"
    state: query
  register: query_with_uuid

- name: Query all L3Out Interface Routing Policies
  cisco.mso.ndo_l3out_interface_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    state: query
  register: query_all

- name: Delete an L3Out Interface Routing Policy
  cisco.mso.ndo_l3out_interface_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: irp_1_updated
    state: absent

- name: Delete an L3Out Interface Routing Policy using UUID
  cisco.mso.ndo_l3out_interface_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ irp_1_present.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, ndo_bfd_multi_hop_settings_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate
from ansible_collections.cisco.mso.plugins.module_utils.constants import ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True),
        uuid=dict(type="str", aliases=["l3out_interface_routing_policy_uuid"]),
        name=dict(type="str", aliases=["l3out_interface_routing_policy_name"]),
        description=dict(type="str"),
        bfd_settings=dict(
            type="dict",
            options=dict(
                state=dict(type="str", choices=["enabled", "disabled"]),
                admin_state=dict(type="str", choices=["enabled", "disabled"]),
                detection_multiplier=dict(type="int"),
                min_receive_interval=dict(type="int"),
                min_transmit_interval=dict(type="int"),
                echo_receive_interval=dict(type="int"),
                echo_admin_state=dict(type="str", choices=["enabled", "disabled"]),
                interface_control=dict(type="str", choices=["enabled", "disabled"]),
            ),
        ),
        bfd_multi_hop_settings=ndo_bfd_multi_hop_settings_spec(),
        ospf_interface_settings=dict(
            type="dict",
            options=dict(
                state=dict(type="str", choices=["enabled", "disabled"]),
                network_type=dict(type="str", choices=["broadcast", "point_to_point"]),
                priority=dict(type="int"),
                cost_of_interface=dict(type="int"),
                advertise_subnet=dict(type="str", choices=["enabled", "disabled"]),
                bfd=dict(type="str", choices=["enabled", "disabled"]),
                mtu_ignore=dict(type="str", choices=["enabled", "disabled"]),
                passive_participation=dict(type="str", choices=["enabled", "disabled"]),
                hello_interval=dict(type="int"),
                dead_interval=dict(type="int"),
                retransmit_interval=dict(type="int"),
                transmit_delay=dict(type="int"),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    bfd_settings = module.params.get("bfd_settings")
    bfd_multi_hop_settings = module.params.get("bfd_multi_hop_settings")
    ospf_interface_settings = module.params.get("ospf_interface_settings")
    state = module.params.get("state")

    mso_template = MSOTemplate(mso, "tenant", template)
    mso_template.validate_template("tenantPolicy")

    l3out_interface_routing_policy = mso_template.get_l3out_interface_routing_policy_object(uuid, name)

    if (uuid or name) and l3out_interface_routing_policy:
        mso.existing = mso.previous = copy.deepcopy(l3out_interface_routing_policy.details)  # Query a specific object
    elif l3out_interface_routing_policy:
        mso.existing = l3out_interface_routing_policy  # Query all objects

    if state != "query":
        interface_routing_policy_path = "/tenantPolicyTemplate/template/l3OutIntfPolGroups/{0}".format(
            l3out_interface_routing_policy.index if l3out_interface_routing_policy else "-"
        )

    ops = []
    if state == "present":
        if mso.existing:
            proposed_payload = copy.deepcopy(mso.existing)
            mso_values = dict()
            mso_values_remove = list()

            mso_values["name"] = name
            mso_values["description"] = description

            # BFD MultiHop Settings
            if bfd_multi_hop_settings is not None:
                if bfd_multi_hop_settings.get("state") == "disabled" and proposed_payload.get("bfdMultiHopPol"):
                    mso_values_remove.append("bfdMultiHopPol")

                elif bfd_multi_hop_settings.get("state") != "disabled":
                    if not proposed_payload.get("bfdMultiHopPol"):
                        mso_values["bfdMultiHopPol"] = dict()

                    mso_values[("bfdMultiHopPol", "adminState")] = bfd_multi_hop_settings.get("admin_state")
                    mso_values[("bfdMultiHopPol", "detectionMultiplier")] = bfd_multi_hop_settings.get("detection_multiplier")
                    mso_values[("bfdMultiHopPol", "minRxInterval")] = bfd_multi_hop_settings.get("min_receive_interval")
                    mso_values[("bfdMultiHopPol", "minTxInterval")] = bfd_multi_hop_settings.get("min_transmit_interval")

            # BFD Settings
            if bfd_settings is not None:
                if bfd_settings.get("state") == "disabled" and proposed_payload.get("bfdPol"):
                    mso_values_remove.append("bfdPol")

                elif bfd_settings.get("state") != "disabled":
                    if not proposed_payload.get("bfdPol"):
                        mso_values["bfdPol"] = dict()

                    mso_values[("bfdPol", "adminState")] = bfd_settings.get("admin_state")
                    mso_values[("bfdPol", "detectionMultiplier")] = bfd_settings.get("detection_multiplier")
                    mso_values[("bfdPol", "minRxInterval")] = bfd_settings.get("min_receive_interval")
                    mso_values[("bfdPol", "minTxInterval")] = bfd_settings.get("min_transmit_interval")
                    mso_values[("bfdPol", "echoRxInterval")] = bfd_settings.get("echo_receive_interval")
                    mso_values[("bfdPol", "echoAdminState")] = bfd_settings.get("echo_admin_state")
                    mso_values[("bfdPol", "ifControl")] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(bfd_settings.get("interface_control"))

            # OSPF Interface Settings
            if ospf_interface_settings is not None:
                if ospf_interface_settings.get("state") == "disabled" and proposed_payload.get("ospfIntfPol"):
                    mso_values_remove.append(("ospfIntfPol",))

                elif ospf_interface_settings.get("state") != "disabled":
                    if not proposed_payload.get("ospfIntfPol"):
                        mso_values[("ospfIntfPol")] = dict()
                        mso_values[("ospfIntfPol", "ifControl")] = dict()

                    mso_values[("ospfIntfPol", "networkType")] = (
                        "pointToPoint" if ospf_interface_settings.get("network_type") == "point_to_point" else ospf_interface_settings.get("network_type")
                    )
                    mso_values[("ospfIntfPol", "prio")] = ospf_interface_settings.get("priority")
                    mso_values[("ospfIntfPol", "cost")] = ospf_interface_settings.get("cost_of_interface")
                    mso_values[("ospfIntfPol", "helloInterval")] = ospf_interface_settings.get("hello_interval")
                    mso_values[("ospfIntfPol", "deadInterval")] = ospf_interface_settings.get("dead_interval")
                    mso_values[("ospfIntfPol", "retransmitInterval")] = ospf_interface_settings.get("retransmit_interval")
                    mso_values[("ospfIntfPol", "transmitDelay")] = ospf_interface_settings.get("transmit_delay")
                    mso_values[("ospfIntfPol", "ifControl", "advertiseSubnet")] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(
                        ospf_interface_settings.get("advertise_subnet")
                    )
                    mso_values[("ospfIntfPol", "ifControl", "bfd")] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(ospf_interface_settings.get("bfd"))
                    mso_values[("ospfIntfPol", "ifControl", "ignoreMtu")] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(
                        ospf_interface_settings.get("mtu_ignore")
                    )
                    mso_values[("ospfIntfPol", "ifControl", "passiveParticipation")] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(
                        ospf_interface_settings.get("passive_participation")
                    )

            append_update_ops_data(ops, proposed_payload, interface_routing_policy_path, mso_values, mso_values_remove)
            mso.sanitize(proposed_payload)
        else:
            mso_values = dict(name=name, description=description)

            # OSPF Interface Settings
            if ospf_interface_settings is not None:
                ospf_interface_pol = dict()
                interface_controls = dict()

                if ospf_interface_settings.get("advertise_subnet"):
                    interface_controls["advertiseSubnet"] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(ospf_interface_settings.get("advertise_subnet"))

                if ospf_interface_settings.get("bfd"):
                    interface_controls["bfd"] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(ospf_interface_settings.get("bfd"))

                if ospf_interface_settings.get("mtu_ignore"):
                    interface_controls["ignoreMtu"] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(ospf_interface_settings.get("mtu_ignore"))

                if ospf_interface_settings.get("passive_participation"):
                    interface_controls["passiveParticipation"] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(
                        ospf_interface_settings.get("passive_participation")
                    )

                if interface_controls:
                    ospf_interface_pol["ifControl"] = interface_controls

                if ospf_interface_settings.get("network_type"):
                    ospf_interface_pol["networkType"] = (
                        "pointToPoint" if ospf_interface_settings.get("network_type") == "point_to_point" else ospf_interface_settings.get("network_type")
                    )

                if ospf_interface_settings.get("priority"):
                    ospf_interface_pol["prio"] = ospf_interface_settings.get("priority")

                if ospf_interface_settings.get("cost_of_interface"):
                    ospf_interface_pol["cost"] = ospf_interface_settings.get("cost_of_interface")

                if ospf_interface_settings.get("hello_interval"):
                    ospf_interface_pol["helloInterval"] = ospf_interface_settings.get("hello_interval")

                if ospf_interface_settings.get("dead_interval"):
                    ospf_interface_pol["deadInterval"] = ospf_interface_settings.get("dead_interval")

                if ospf_interface_settings.get("retransmit_interval"):
                    ospf_interface_pol["retransmitInterval"] = ospf_interface_settings.get("retransmit_interval")

                if ospf_interface_settings.get("transmit_delay"):
                    ospf_interface_pol["transmitDelay"] = ospf_interface_settings.get("transmit_delay")

                if ospf_interface_pol or ospf_interface_settings.get("state") == "enabled":
                    mso_values["ospfIntfPol"] = ospf_interface_pol

            # BFD MultiHop Settings
            if bfd_multi_hop_settings is not None:
                bfd_multi_hop_pol = dict()
                if bfd_multi_hop_settings.get("admin_state"):
                    bfd_multi_hop_pol["adminState"] = bfd_multi_hop_settings.get("admin_state")

                if bfd_multi_hop_settings.get("detection_multiplier"):
                    bfd_multi_hop_pol["detectionMultiplier"] = bfd_multi_hop_settings.get("detection_multiplier")

                if bfd_multi_hop_settings.get("min_receive_interval"):
                    bfd_multi_hop_pol["minRxInterval"] = bfd_multi_hop_settings.get("min_receive_interval")

                if bfd_multi_hop_settings.get("min_transmit_interval"):
                    bfd_multi_hop_pol["minTxInterval"] = bfd_multi_hop_settings.get("min_transmit_interval")

                if bfd_multi_hop_pol or bfd_multi_hop_settings.get("state") == "enabled":
                    mso_values["bfdMultiHopPol"] = bfd_multi_hop_pol

            # BFD Settings
            if bfd_settings is not None:
                bfd_settings_map = dict()

                if bfd_settings.get("admin_state"):
                    bfd_settings_map["adminState"] = bfd_settings.get("admin_state")

                if bfd_settings.get("detection_multiplier"):
                    bfd_settings_map["detectionMultiplier"] = bfd_settings.get("detection_multiplier")

                if bfd_settings.get("min_receive_interval"):
                    bfd_settings_map["minRxInterval"] = bfd_settings.get("min_receive_interval")

                if bfd_settings.get("min_transmit_interval"):
                    bfd_settings_map["minTxInterval"] = bfd_settings.get("min_transmit_interval")

                if bfd_settings.get("echo_receive_interval"):
                    bfd_settings_map["echoRxInterval"] = bfd_settings.get("echo_receive_interval")

                if bfd_settings.get("echo_admin_state"):
                    bfd_settings_map["echoAdminState"] = bfd_settings.get("echo_admin_state")

                if bfd_settings.get("interface_control"):
                    bfd_settings_map["ifControl"] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(bfd_settings.get("interface_control"))

                if bfd_settings_map or bfd_settings.get("state") == "enabled":
                    mso_values["bfdPol"] = bfd_settings_map

            ops.append(dict(op="add", path=interface_routing_policy_path, value=copy.deepcopy(mso_values)))

            mso.sanitize(mso_values)
    elif state == "absent":
        if mso.existing:
            ops.append(dict(op="remove", path=interface_routing_policy_path))

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        l3out_interface_routing_policy = mso_template.get_l3out_interface_routing_policy_object(uuid, name)
        if l3out_interface_routing_policy:
            mso.existing = l3out_interface_routing_policy.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: GPL-3.0-only
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_dhcp_lease
short_description: Manage Windows Server DHCP Leases
author: Joe Zollo (@joezollo)
requirements:
  - This module requires Windows Server 2012 or Newer
description:
  - Manage Windows Server DHCP Leases (IPv4 Only)
  - Adds, Removes and Modifies DHCP Leases and Reservations
  - Task should be delegated to a Windows DHCP Server
version_added: 2.6.0
options:
  type:
    description:
      - The type of DHCP address.
      - Leases expire as defined by l(duration).
      - When l(duration) is not specified, the server default is used.
      - Reservations are permanent.
    type: str
    default: reservation
    choices: [ reservation, lease ]
  state:
    description:
      - Specifies the desired state of the DHCP lease or reservation.
    type: str
    default: present
    choices: [ present, absent ]
  ip:
    description:
      - The IPv4 address of the client server/computer.
      - This is a required parameter, if l(mac) is not set.
      - Can be used to identify an existing lease/reservation, instead of l(mac).
    type: str
    required: no
  scope_id:
    description:
      - Specifies the scope identifier as defined by the DHCP server.
      - This is a required parameter, if l(state=present) and the reservation or lease
        doesn't already exist. Not required if updating an existing lease or reservation.
    type: str
  mac:
    description:
      - Specifies the client identifier to be set on the IPv4 address.
      - This is a required parameter, if l(ip) is not set.
      - Windows clients use the MAC address as the client ID.
      - Linux and other operating systems can use other types of identifiers.
      - Can be used to identify an existing lease/reservation, instead of l(ip).
    type: str
  duration:
    description:
      - Specifies the duration of the DHCP lease in days.
      - The duration value only applies to l(type=lease).
      - Defaults to the duration specified by the DHCP server
        configuration.
      - Only applicable to l(type=lease).
    type: int
  dns_hostname:
    description:
      - Specifies the DNS hostname of the client for which the IP address
        lease is to be added.
    type: str
  dns_regtype:
    description:
      - Indicates the type of DNS record to be registered by the DHCP.
        server service for this lease.
      - l(a) results in an A record being registered.
      - l(aptr) results in both A and PTR records to be registered.
      - l(noreg) results in no DNS records being registered.
    type: str
    default: aptr
    choices: [ aptr, a, noreg ]
  reservation_name:
    description:
      - Specifies the name of the reservation being created.
      - Only applicable to l(type=reservation).
    type: str
  description:
    description:
      - Specifies the description for reservation being created.
      - Only applicable to l(type=reservation).
    type: str
  computer_name:
    description:
      - Specifies a DHCP server.
      - You can specify an IP address or any value that resolves to an IP
        address, such as a fully qualified domain name (FQDN), host name, or
        NETBIOS name.
    type: str
    version_added: 3.6.0

'''

EXAMPLES = r'''
- name: Ensure DHCP reservation exists
  ansible.windows.win_dhcp_lease:
    type: reservation
    ip: 192.168.100.205
    scope_id: 192.168.100.0
    mac: 00:B1:8A:D1:5A:1F
    dns_hostname: "{{ ansible_inventory }}"
    description: Testing Server

- name: Ensure DHCP lease or reservation does not exist
  ansible.windows.win_dhcp_lease:
    mac: 00:B1:8A:D1:5A:1F
    state: absent

- name: Ensure DHCP lease or reservation does not exist
  ansible.windows.win_dhcp_lease:
    ip: 192.168.100.205
    state: absent

- name: Convert DHCP lease to reservation & update description
  ansible.windows.win_dhcp_lease:
    type: reservation
    ip: 192.168.100.205
    description: Testing Server

- name: Convert DHCP reservation to lease
  ansible.windows.win_dhcp_lease:
    type: lease
    ip: 192.168.100.205

# Modify an existing lease by running the following tasks
- name: Remove old lease
  win_dhcp_lease:
    mac: "00:11:22:33:44:55"
    state: absent

- name: Create new lease with updated properties
  win_dhcp_lease:
    mac: "00:11:22:33:44:55"
    ip: "192.168.100.100"
    duration: 14
    state: present
'''

RETURN = r'''
lease:
  description: New/Updated DHCP object parameters
  returned: When l(state=present)
  type: dict
  sample:
    address_state: InactiveReservation
    client_id: 0a-0b-0c-04-05-aa
    description: Really Fancy
    ip_address: 172.16.98.230
    name: null
    scope_id: 172.16.98.0
'''



# Copyright (c) 2024, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.crypto.plugins.module_utils._crypto.math import (
    convert_int_to_hex,
)


def th(number: int) -> str:
    abs_number = abs(number)
    mod_10 = abs_number % 10
    mod_100 = abs_number % 100
    if mod_100 not in (11, 12, 13):
        if mod_10 == 1:
            return "st"
        if mod_10 == 2:
            return "nd"
        if mod_10 == 3:
            return "rd"
    return "th"


def parse_serial(value: str | bytes) -> int:
    """
    Given a colon-separated string of hexadecimal byte values, converts it to an integer.
    """
    value_str = to_text(value)
    result = 0
    for i, part in enumerate(value_str.split(":")):
        try:
            part_value = int(part, 16)
            if part_value < 0 or part_value > 255:
                raise ValueError("the value is not in range [0, 255]")
        except ValueError as exc:
            raise ValueError(
                f"The {i + 1}{th(i + 1)} part {part!r} is not a hexadecimal number in range [0, 255]: {exc}"
            ) from exc
        result = (result << 8) | part_value
    return result


def to_serial(value: int) -> str:
    """
    Given an integer, converts its absolute value to a colon-separated string of hexadecimal byte values.
    """
    value_str = convert_int_to_hex(value).upper()
    if len(value_str) % 2 != 0:
        value_str = f"0{value_str}"
    return ":".join(value_str[i : i + 2] for i in range(0, len(value_str), 2))


__all__ = ("parse_serial", "to_serial")



# Copyright (c) 2016 Matt Clay <matt@mystile.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Matt Clay (@mattclay) <matt@mystile.com>
name: lxd
short_description: Run tasks in LXD instances using C(lxc) CLI
description:
  - Run commands or put/fetch files to an existing instance using C(lxc) CLI.
options:
  remote_addr:
    description:
      - Instance (container/VM) identifier.
      - Since community.general 8.0.0, a FQDN can be provided; in that case, the first component (the part before C(.)) is
        used as the instance identifier.
    type: string
    default: inventory_hostname
    vars:
      - name: inventory_hostname
      - name: ansible_host
      - name: ansible_lxd_host
  executable:
    description:
      - Shell to use for execution inside instance.
    type: string
    default: /bin/sh
    vars:
      - name: ansible_executable
      - name: ansible_lxd_executable
  lxd_become_method:
    description:
      - Become command used to switch to a non-root user.
      - Is only used when O(remote_user) is not V(root).
    type: str
    default: /bin/su
    vars:
      - name: lxd_become_method
    version_added: 10.4.0
  remote:
    description:
      - Name of the LXD remote to use.
    type: string
    default: local
    vars:
      - name: ansible_lxd_remote
    version_added: 2.0.0
  remote_user:
    description:
      - User to login/authenticate as.
      - Can be set from the CLI with the C(--user) or C(-u) options.
    type: string
    default: root
    vars:
      - name: ansible_user
    env:
      - name: ANSIBLE_REMOTE_USER
    ini:
      - section: defaults
        key: remote_user
    keyword:
      - name: remote_user
    version_added: 10.4.0
  project:
    description:
      - Name of the LXD project to use.
    type: string
    vars:
      - name: ansible_lxd_project
    version_added: 2.0.0
"""

import os
from subprocess import PIPE, Popen

from ansible.errors import AnsibleConnectionFailure, AnsibleError, AnsibleFileNotFound
from ansible.module_utils.common.process import get_bin_path
from ansible.module_utils.common.text.converters import to_bytes, to_text
from ansible.plugins.connection import ConnectionBase


class Connection(ConnectionBase):
    """lxd based connections"""

    transport = "community.general.lxd"
    has_pipelining = True

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super().__init__(play_context, new_stdin, *args, **kwargs)

        try:
            self._lxc_cmd = get_bin_path("lxc")
        except ValueError as e:
            raise AnsibleError("lxc command not found in PATH") from e

    def _host(self):
        """translate remote_addr to lxd (short) hostname"""
        return self.get_option("remote_addr").split(".", 1)[0]

    def _connect(self):
        """connect to lxd (nothing to do here)"""
        super()._connect()

        if not self._connected:
            self._display.vvv(f"ESTABLISH LXD CONNECTION FOR USER: {self.get_option('remote_user')}", host=self._host())
            self._connected = True

    def _build_command(self, cmd) -> list[str]:
        """build the command to execute on the lxd host"""

        exec_cmd: list[str] = [self._lxc_cmd]

        if self.get_option("project"):
            exec_cmd.extend(["--project", self.get_option("project")])

        exec_cmd.extend(["exec", f"{self.get_option('remote')}:{self._host()}", "--"])

        if self.get_option("remote_user") != "root":
            self._display.vvv(
                f"INFO: Running as non-root user: {self.get_option('remote_user')}, \
                trying to run 'lxc exec' with become method: {self.get_option('lxd_become_method')}",
                host=self._host(),
            )
            exec_cmd.extend([self.get_option("lxd_become_method"), self.get_option("remote_user"), "-c"])

        exec_cmd.extend([self.get_option("executable"), "-c", cmd])

        return exec_cmd

    def exec_command(self, cmd, in_data=None, sudoable=True):
        """execute a command on the lxd host"""
        super().exec_command(cmd, in_data=in_data, sudoable=sudoable)

        self._display.vvv(f"EXEC {cmd}", host=self._host())

        local_cmd = self._build_command(cmd)
        self._display.vvvvv(f"EXEC {local_cmd}", host=self._host())

        local_cmd = [to_bytes(i, errors="surrogate_or_strict") for i in local_cmd]
        in_data = to_bytes(in_data, errors="surrogate_or_strict", nonstring="passthru")

        process = Popen(local_cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        stdout, stderr = process.communicate(in_data)

        stdout = to_text(stdout)
        stderr = to_text(stderr)

        self._display.vvvvv(f"EXEC lxc output: {stdout} {stderr}", host=self._host())

        if "is not running" in stderr:
            raise AnsibleConnectionFailure(f"instance not running: {self._host()}")

        if stderr.strip() == "Error: Instance not found" or stderr.strip() == "error: not found":
            raise AnsibleConnectionFailure(f"instance not found: {self._host()}")

        return process.returncode, stdout, stderr

    def _get_remote_uid_gid(self) -> tuple[int, int]:
        """Get the user and group ID of 'remote_user' from the instance."""

        rc, uid_out, err = self.exec_command("/bin/id -u")
        if rc != 0:
            raise AnsibleError(f"Failed to get remote uid for user {self.get_option('remote_user')}: {err}")
        uid = uid_out.strip()

        rc, gid_out, err = self.exec_command("/bin/id -g")
        if rc != 0:
            raise AnsibleError(f"Failed to get remote gid for user {self.get_option('remote_user')}: {err}")
        gid = gid_out.strip()

        return int(uid), int(gid)

    def put_file(self, in_path, out_path):
        """put a file from local to lxd"""
        super().put_file(in_path, out_path)

        self._display.vvv(f"PUT {in_path} TO {out_path}", host=self._host())

        if not os.path.isfile(to_bytes(in_path, errors="surrogate_or_strict")):
            raise AnsibleFileNotFound(f"input path is not a file: {in_path}")

        local_cmd = [self._lxc_cmd]
        if self.get_option("project"):
            local_cmd.extend(["--project", self.get_option("project")])

        if self.get_option("remote_user") != "root":
            uid, gid = self._get_remote_uid_gid()
            local_cmd.extend(
                [
                    "file",
                    "push",
                    "--uid",
                    str(uid),
                    "--gid",
                    str(gid),
                    in_path,
                    f"{self.get_option('remote')}:{self._host()}/{out_path}",
                ]
            )
        else:
            local_cmd.extend(
                [
                    "file",
                    "push",
                    in_path,
                    f"{self.get_option('remote')}:{self._host()}/{out_path}",
                ]
            )

        self._display.vvvvv(f"PUT {local_cmd}", host=self._host())

        local_cmd = [to_bytes(i, errors="surrogate_or_strict") for i in local_cmd]

        process = Popen(local_cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        process.communicate()

    def fetch_file(self, in_path, out_path):
        """fetch a file from lxd to local"""
        super().fetch_file(in_path, out_path)

        self._display.vvv(f"FETCH {in_path} TO {out_path}", host=self._host())

        local_cmd = [self._lxc_cmd]
        if self.get_option("project"):
            local_cmd.extend(["--project", self.get_option("project")])
        local_cmd.extend(["file", "pull", f"{self.get_option('remote')}:{self._host()}/{in_path}", out_path])

        local_cmd = [to_bytes(i, errors="surrogate_or_strict") for i in local_cmd]

        process = Popen(local_cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        process.communicate()

    def close(self):
        """close the connection (nothing to do here)"""
        super().close()

        self._connected = False



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Vrf_address_family parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Vrf_address_familyTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Vrf_address_familyTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "address-family {{ afi }} {{ safi }}",
            "remval": "no address-family {{ afi }} {{ safi }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                        },
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "export.map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+export\smap\s(?P<export_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "export map {{ export.map }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "export": {
                                "map": "{{ export_map }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "import_config.map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+import\smap\s(?P<import_config_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "import map {{ import_config.map }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "import_config": {
                                "map": "{{ import_config_map }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "export.ipv4.multicast",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+export\sipv4\smulticast\s(?P<prefix>\d+)\smap\s(?P<export_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "export ipv4 multicast {{ export.ipv4.multicast.prefix }} "
                "map {{ export.ipv4.multicast.prefix.map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "export": {
                                "ipv4": {
                                    "multicast": {
                                        "prefix": "{{ prefix }}",
                                        "map": "{{ export_map }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "export.ipv4.unicast.allow_evpn",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+export\sipv4\sunicast\s(?P<prefix>\d+)\smap\s(?P<export_map>\S+)\s(?P<allow_evpn>allow-evpn)
                $""", re.VERBOSE,
            ),
            "setval": (
                "export ipv4 unicast {{ export.ipv4.unicast.prefix }} map "
                "{{ export.ipv4.unicast.map.export_map }} "
                "allow-evpn"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "export": {
                                "ipv4": {
                                    "unicast": {
                                        "prefix": "{{ prefix }}",
                                        "map": "{{ export_map }}",
                                        "allow_evpn": (
                                            "{{true if allow_evpn is defined}}"
                                        ),
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "import_config.ipv4.multicast",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+import\sipv4\smulticast\s(?P<prefix>\d+)\smap\s(?P<import_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "import ipv4 multicast {{ prefix }} map {{ import_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "import_config": {
                                "ipv4": {
                                    "multicast": {
                                        "prefix": "{{ prefix }}",
                                        "map": "{{ import_map }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "import_config.ipv4.unicast",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+import\sipv4\sunicast\s(?P<limit>\d+)\smap\s(?P<import_map>\S+)\s(?P<allow_evpn>allow-evpn)
                $""", re.VERBOSE,
            ),
            "setval": (
                "import ipv4 unicast {{ limit }} map {{ import_map }} "
                "allow-evpn"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "import_config": {
                                "ipv4": {
                                    "unicast": {
                                        "limit": "{{ limit }}",
                                        "map": "{{ import_map }}",
                                        "allow_evpn": (
                                            "{{true if allow_evpn is defined}}"
                                        ),
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp.next_hop.loopback",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+bgp\snext-hop\sLoopback(?P<loopback>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp next-hop Loopback {{ loopback }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "bgp": {
                                "next_hop": {
                                    "loopback": "{{ loopback }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "inter_as_hybrid.csc.next_hop",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+inter-as-hybrid\scsc\snext-hop\s(?P<inter_as_hybrid_csc_next_hop>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "inter-as-hybrid csc next-hop "
                "{{ inter_as_hybrid.csc.next_hop }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "inter_as_hybrid": {
                                "csc": {
                                    "next_hop": (
                                        "{{ inter_as_hybrid_csc_next_hop }}"
                                    ),
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "inter_as_hybrid.next_hop",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+inter-as-hybrid\snext-hop\s(?P<inter_as_hybrid_next_hop>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "inter-as-hybrid next-hop {{ inter_as_hybrid.next_hop }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "inter_as_hybrid": {
                                "next_hop": "{{ inter_as_hybrid_next_hop }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": (
                "mdt.auto_discovery.ingress_replication.inter_as."
                "mdt_hello_enable"
            ),
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sauto-discovery\singress-replication\sinter-as\smdt-hello-enable
                $""", re.VERBOSE,
            ),
            "setval": (
                "mdt auto-discovery ingress-replication inter-as "
                "mdt-hello-enable"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "auto_discovery": {
                                    "ingress_replication": {
                                        "inter_as": {
                                            "mdt_hello_enable": "{{ true }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.auto_discovery.pim.inter_as.mdt_hello_enable",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sauto-discovery\spim\sinter-as\smdt-hello-enable
                $""", re.VERBOSE,
            ),
            "setval": "mdt auto-discovery pim inter-as mdt-hello-enable",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "auto_discovery": {
                                    "pim": {
                                        "inter_as": {
                                            "mdt_hello_enable": "{{ true }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": (
                "mdt.auto_discovery.pim.inter_as.pim_tlv_announce."
                "mdt_hello_enable"
            ),
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sauto-discovery\spim\sinter-as\spim-tlv-announce\smdt-hello-enable
                $""", re.VERBOSE,
            ),
            "setval": (
                "mdt auto-discovery pim inter-as pim-tlv-announce "
                "mdt-hello-enable"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "auto_discovery": {
                                    "pim": {
                                        "inter_as": {
                                            "pim_tlv_announce": {
                                                "mdt_hello_enable": "{{true}}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.auto_discovery.ingress_replication.mdt_hello_enable",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sauto-discovery\singress-replication\smdt-hello-enable
                $""", re.VERBOSE,
            ),
            "setval": (
                "mdt auto-discovery ingress-replication mdt-hello-enable"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "auto_discovery": {
                                    "ingress_replication": {
                                        "mdt_hello_enable": "{{ true }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.auto_discovery.pim.mdt_hello_enable",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sauto-discovery\spim\smdt-hello-enable
                $""", re.VERBOSE,
            ),
            "setval": "mdt auto-discovery pim mdt-hello-enable",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "auto_discovery": {
                                    "pim": {
                                        "mdt_hello_enable": "{{ true }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.auto_discovery.pim.pim_tlv_announce.mdt_hello_enable",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sauto-discovery\spim\spim-tlv-announce\smdt-hello-enable
                $""", re.VERBOSE,
            ),
            "setval": (
                "mdt auto-discovery pim pim-tlv-announce mdt-hello-enable"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "auto_discovery": {
                                    "pim": {
                                        "pim_tlv_announce": {
                                            "mdt_hello_enable": "{{ true }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.auto_discovery.mldp",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sauto-discovery\smldp
                $""", re.VERBOSE,
            ),
            "setval": "mdt auto-discovery mldp",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "auto_discovery": {
                                    "mldp": "{{ true }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.auto_discovery.receiver_site",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sauto-discovery\sreceiver-site
                $""", re.VERBOSE,
            ),
            "setval": "mdt auto-discovery receiver-site",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "auto_discovery": {
                                    "receiver_site": "{{ true }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.data.ingress_replication.number",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sdata\singress-replication\s(?P<mdt_data_ingress_replication_number>\d+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "mdt data ingress-replication "
                "{{ mdt.data.ingress_replication.number }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "data": {
                                    "ingress_replication": {
                                        "number": (
                                            "{{ mdt_data_ingress_"
                                            "replication_number }}"
                                        ),
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.data.ingress_replication.immediate_switch",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sdata\singress-replication\simmediate-switch
                $""", re.VERBOSE,
            ),
            "setval": "mdt data ingress-replication immediate-switch",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "data": {
                                    "ingress_replication": {
                                        "immediate_switch": "{{ true }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.data.ingress_replication.number.immediate_switch",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sdata\singress-replication\s(?P<mdt_data_ingress_replication_number>\d+)\simmediate-switch
                $""", re.VERBOSE,
            ),
            "setval": (
                "mdt data ingress-replication "
                "{{ mdt.data.ingress_replication.number }} immediate-switch"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "data": {
                                    "ingress_replication": {
                                        "number": (
                                            "{{ mdt_data_ingress_"
                                            "replication_number }}"
                                        ),
                                        "immediate_switch": "{{ true }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.data.mpls.mldp",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sdata\smpls\smldp\s(?P<mldp_val>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "mdt data mpls mldp {{ mdt.data.mpls.mldp }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "data": {
                                    "mpls": {
                                        "mldp": (
                                            "{{ mldp_val }}"
                                        ),
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.data.list.access_list",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sdata\slist\s(?P<mdt_data_list_access_list>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "mdt data list {{ mdt.data.list.access_list }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "data": {
                                    "list": {
                                        "access_list": (
                                            "{{ mdt_data_list_access_list }}"
                                        ),
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.data.list.access_list_name",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sdata\slist\s(?P<mdt_data_list_access_list_name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "mdt data list {{ mdt.data.list.access_list_name }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "data": {
                                    "list": {
                                        "access_list_name": (
                                            "{{ mdt_data_list_access_list_"
                                            "name }}"
                                        ),
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.data.threshold",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sdata\sthreshold\s(?P<mdt_data_threshold>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "mdt data threshold {{ mdt.data.threshold }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "data": {
                                    "threshold": "{{ mdt_data_threshold }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.default_ingress_replication",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sdefault\singress-replication
                $""", re.VERBOSE,
            ),
            "setval": "mdt default ingress-replication",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "default": {
                                    "ingress_replication": "{{ true }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.direct",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sdirect
                $""", re.VERBOSE,
            ),
            "setval": "mdt direct",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "direct": "{{ true }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.log_reuse",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\slog-reuse
                $""", re.VERBOSE,
            ),
            "setval": "mdt log-reuse",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "log_reuse": "{{ true }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.mode.gre",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\smode\sgre
                $""", re.VERBOSE,
            ),
            "setval": "mdt mode gre",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "mode": {
                                    "gre": "{{ true }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.mtu.value",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\smtu\s(?P<mtu_value>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "mdt mtu {{ mdt.mtu.value }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "mtu": "{{ mdt_value }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.overlay.bgp.shared_tree_prune_delay",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\soverlay\sbgp\sshared-tree-prune-delay\s(?P<shared_tree_prune_delay>\d+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "mdt overlay bgp shared-tree-prune-delay "
                "{{ mdt.overlay.bgp.shared_tree_prune_delay }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "overlay": {
                                    "bgp": {
                                        "shared_tree_prune_delay": (
                                            "{{ shared_tree_prune_delay }}"
                                        ),
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.overlay.bgp.source_tree_prune_delay",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\soverlay\sbgp\ssource-tree-prune-delay\s(?P<source_tree_prune_delay>\d+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "mdt overlay bgp source-tree-prune-delay "
                "{{ mdt.overlay.bgp.source_tree_prune_delay }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "overlay": {
                                    "bgp": {
                                        "source_tree_prune_delay": (
                                            "{{ source_tree_prune_delay }}"
                                        ),
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.overlay.use_bgp_spt_only",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\soverlay\suse-bgp\sspt-only
                $""", re.VERBOSE,
            ),
            "setval": "mdt overlay use-bgp spt-only",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "overlay": {
                                    "use_bgp": {
                                        "spt_only": "{{ true }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.overlay.use_bgp",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\soverlay\suse-bgp
                $""", re.VERBOSE,
            ),
            "setval": "mdt overlay use-bgp",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "overlay": {
                                    "use_bgp": {
                                        "set": "{{ true }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.partitioned.mldp.p2mp",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\spartitioned\smldp\sp2mp
                $""", re.VERBOSE,
            ),
            "setval": "mdt partitioned mldp p2mp",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "partitioned": {
                                    "mldp": {
                                        "p2mp": "{{ true }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.partitioned.ingress_replication",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\spartitioned\singress-replication
                $""", re.VERBOSE,
            ),
            "setval": "mdt partitioned ingress-replication",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "partitioned": {
                                    "ingress_replication": "{{ true }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.strict_rpf_interface",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sstrict-rpf\sinterface
                $""", re.VERBOSE,
            ),
            "setval": "mdt strict-rpf interface",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "mdt": {
                                "strict_rpf": {
                                    "interface": "{{ true }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "protection.local_prefixes",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "protection local-prefixes",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "protection": {
                                "local_prefixes": "{{ true }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.recursion_policy.destination",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\srecursion-policy\sdestination
                $""", re.VERBOSE,
            ),
            "setval": "route-replicate recursion-policy destination",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "recursion_policy": {
                                    "destination": "{{ true }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.vrf.vrf_name.unicast.all.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\sall\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} unicast all "
                "route-map {{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "all": {
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": (
                "route_replicate.from.vrf.vrf_name.unicast.bgp.asn.route_map"
            ),
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\sbgp\s(?P<asn>\d+)\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} "
                "unicast bgp {{ asn }} "
                "route-map {{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "bgp": {
                                                "as_number": "{{ asn }}",
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": (
                "route_replicate.from.vrf.vrf_name.unicast.connected.route_map"
            ),
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\sconnected\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} unicast connected "
                "route-map {{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "connected": {
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": (
                "route_replicate.from.vrf.vrf_name.unicast.eigrp.asn."
                "route_map"
            ),
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\seigrp\s(?P<asn>\d+)\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} "
                "unicast eigrp {{ asn }} "
                "route-map {{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "eigrp": {
                                                "as_number": "{{ asn }}",
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.vrf.vrf_name.unicast.isis.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\sisis\s(?P<tag>\S+)\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} unicast isis "
                "{{ tag }} route-map {{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "isis": {
                                                "iso_tag": "{{ tag }}",
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": (
                "route_replicate.from.vrf.vrf_name.unicast.mobile.route_map"
            ),
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\smobile\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} unicast mobile "
                "route-map {{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "mobile": {
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.vrf.vrf_name.unicast.odr.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\sodr\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} "
                "unicast odr route-map {{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "odr": {
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": (
                "route_replicate.from.vrf.vrf_name.unicast.ospf.id."
                "route_map"
            ),
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\sospf\s(?P<process_id>\d+)\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} unicast ospf "
                "{{ process_id }} route-map "
                "{{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "ospf": {
                                                "process_id": (
                                                    "{{ process_id }}"
                                                ),
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.vrf.vrf_name.unicast.rip.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\srip\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} "
                "unicast rip route-map {{ route_replicate.from.vrf.vrf_name."
                "unicast.rip.route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "rip": {
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": (
                "route_replicate.from.vrf.vrf_name.unicast.static.route_map"
            ),
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\svrf\s(?P<vrf_name>\S+)\sunicast\sstatic\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from vrf {{ vrf_name }} "
                "unicast static route-map {{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "vrf": {
                                        "name": "{{ vrf_name }}",
                                        "unicast": {
                                            "static": {
                                                "route_map": "{{ route_map }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.all.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\sall\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast all route-map "
                "{{ route_replicate.from.unicast.all.route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "all": {
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.bgp.asn.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\sbgp\s(?P<asn>\d+)\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast bgp "
                "{{ route_replicate.from.unicast.bgp.asn }} "
                "route-map {{ route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "bgp": {
                                            "as_number": "{{ asn }}",
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.connected.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\sconnected\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast connected route-map "
                "{{ route_replicate.from.unicast.connected.route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "connected": {
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.eigrp.asn.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\seigrp\s(?P<asn>\d+)\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast eigrp "
                "{{ route_replicate.from.unicast.eigrp.asn }} "
                "route-map {{ route_replicate.from.unicast.eigrp.asn."
                "route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "eigrp": {
                                            "as_number": "{{ asn }}",
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.isis.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\sisis\s(?P<tag>\S+)\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast isis "
                "{{ route_replicate.from.unicast.isis.tag }} "
                "route-map {{ route_replicate.from.unicast.isis.route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "isis": {
                                            "tag": "{{ tag }}",
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.mobile.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\smobile\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast mobile route-map "
                "{{ route_replicate.from.unicast.mobile.route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "mobile": {
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.odr.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\sodr\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast odr route-map "
                "{{ route_replicate.from.unicast.odr.route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "odr": {
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.ospf.id.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\sospf\s(?P<process_id>\d+)\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast ospf "
                "{{ route_replicate.from.unicast.ospf.id }} "
                "route-map {{ route_replicate.from.unicast.ospf.id.route_map}}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "ospf": {
                                            "process_id": "{{ process_id }}",
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.rip.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\srip\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast rip route-map "
                "{{ route_replicate.from.unicast.rip.route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "rip": {
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_replicate.from.unicast.static.route_map",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-replicate\sfrom\sunicast\sstatic\sroute-map\s(?P<route_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": (
                "route-replicate from unicast static route-map "
                "{{ route_replicate.from.unicast.static.route_map }}"
            ),
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": (
                                "{{ safi if safi is defined else "
                                "'unicast' }}"
                            ),
                            "route_replicate": {
                                "from_config": {
                                    "unicast": {
                                        "static": {
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_target.exports",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-target\sexport\s(?P<route_target_export>\S+)
                (\s(?P<stitching_export>stitching))?
                $""", re.VERBOSE,
            ),
            "setval": "route-target export {{ rt_value }}{% if stitching %} stitching{% endif %}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + ("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi if safi is defined else 'unicast' }}",
                            "route_target": {
                                "exports": [
                                    {
                                        "rt_value": "{{ route_target_export }}",
                                        "stitching": "{{ True if stitching_export is defined else False }}",
                                    },
                                ],
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_target.imports",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+route-target\simport\s(?P<route_target_import>\S+)
                (\s(?P<stitching_import>stitching))?
                $""", re.VERBOSE,
            ),
            "setval": "route-target import {{ rt_value }}{% if stitching %} stitching{% endif %}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + ("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi if safi is defined else 'unicast' }}",
                            "route_target": {
                                "imports": [
                                    {
                                        "rt_value": "{{ route_target_import }}",
                                        "stitching": "{{ True if stitching_import is defined else False }}",
                                    },
                                ],
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mdt.strict_rpf.interface",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition\s(?P<name>\S+)
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                \s+mdt\sstrict-rpf\sinterface
                $""", re.VERBOSE,
            ),
            "setval": "mdt strict-rpf interface",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{ "address_families_" + afi + '
                        '("_" + safi if safi is defined else "_unicast") }}': {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi if safi is defined else 'unicast' }}",
                            "mdt": {
                                "strict_rpf": {"interface": "{{ true }}"},
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_imc_access_policy_info
short_description: Gather information about IMC Access Policies
description:
  - Gathers information about IMC Access Policies on Cisco Intersight.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the Organization this resource is assigned to.
    type: str
    default: default
  name:
    description:
      - The name assigned to the IMC Access Policy.
    type: str
author:
  - Steve Fulmer (@sfulmer)
'''

EXAMPLES = r'''
- name: Get IMC Access Policy Info
  cisco.intersight.intersight_imc_access_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "imc-access-policy"
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: list
  sample:
    [
      {
        "Name": "imc-access-policy",
        "ObjectType": "access.Policy",
        "AddressType": {}
      }
    ]
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str', default='default'),
        name=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/access/Policies'
    query_params = intersight.set_query_params()

    intersight.get_resource(resource_path=resource_path, query_params=query_params, return_list=True)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import typing

if typing.TYPE_CHECKING:
    from typing import Dict
    from typing import List

    from ..transformation import AnsibleAWSResource
    from ..transformation import AnsibleAWSResourceList
    from ..transformation import BotoResource
    from ..transformation import BotoResourceList

from ..tagging import boto3_tag_list_to_ansible_dict
from ..transformation import boto3_resource_list_to_ansible_dict
from ..transformation import boto3_resource_to_ansible_dict


def _normalize_attributes(attributes: BotoResourceList) -> Dict[str, str]:
    """
    Normalize ELB load balancer attributes from boto3 format to Ansible format.

    Converts attributes from the list of Key/Value dicts returned by
    describe_load_balancer_attributes to a flat dict with underscored keys.

    Parameters:
        attributes (list): List of attribute dicts from boto3, format:
            [{"Key": "access_logs.s3.enabled", "Value": "true"}, ...]

    Returns:
        Dict with underscored keys and string values:
            {"access_logs_s3_enabled": "true", ...}
    """
    if not attributes:
        return {}

    # Convert from boto3 format [{"Key": "...", "Value": "..."}] to dict
    attrs_dict = boto3_tag_list_to_ansible_dict(attributes)

    # Replace '.' with '_' in attribute keys to make it more Ansibley
    return {k.replace(".", "_"): v for k, v in attrs_dict.items()}


def _sort_actions(actions: BotoResourceList) -> BotoResourceList:
    """
    Sort ELB listener actions by Order and Type.

    Sorts actions by Order field (defaulting to 0), then by Type for stability.
    This ensures consistent ordering and avoids comparing dict/None which causes TypeError.

    Parameters:
        actions (list): List of action dicts from boto3

    Returns:
        List of actions sorted by Order and Type
    """
    if not actions:
        return actions

    return sorted(
        actions,
        key=lambda x: (
            x.get("Order", 0),
            x.get("Type", ""),
        ),
    )


def _normalize_listener_actions(actions: BotoResourceList) -> AnsibleAWSResourceList:
    """
    Normalize and sort ELB listener actions by Order and Type.

    Converts actions from the CamelCase boto3 format to the snake_case Ansible format
    and sorts them by Order (defaulting to 0) then Type for stability.

    Parameters:
        actions (list): List of listener actions from boto3

    Returns:
        List of actions in Ansible format, sorted by Order and Type
    """
    if not actions:
        return actions

    # Sort actions before conversion
    sorted_actions = _sort_actions(actions)

    # Convert to Ansible dict format (force_tags=False to prevent automatic tags field)
    return boto3_resource_list_to_ansible_dict(sorted_actions, force_tags=False)


def _normalize_listener_rules(rules: BotoResourceList) -> AnsibleAWSResourceList:
    """
    Normalize and sort ELB listener rules by priority.

    Converts listener rules from the CamelCase boto3 format to the snake_case Ansible format
    and sorts them numerically by priority, with the "default" rule appearing last.
    Actions within each rule are also sorted by Order and Type.

    Parameters:
        rules (list): List of listener rules from boto3

    Returns:
        List of rules in Ansible format, sorted by priority with sorted actions
    """
    if not rules:
        return rules

    # First sort the rules by Priority (while still in CamelCase)
    def sort_key(rule):
        priority = rule.get("Priority", "default")
        if priority == "default":
            # Put default rule last by using a very large number
            return float("inf")
        try:
            return int(priority)
        except (ValueError, TypeError):
            # Fallback for unexpected priority values
            return float("inf")

    sorted_rules = sorted(rules, key=sort_key)

    # Convert to Ansible dict format, also sorting Actions within each rule
    # (force_tags=False to prevent automatic tags field)
    transforms = {"Actions": _normalize_listener_actions}
    return [
        boto3_resource_to_ansible_dict(rule, nested_transforms=transforms, force_tags=False) for rule in sorted_rules
    ]


def _normalize_listeners(listeners: BotoResourceList) -> AnsibleAWSResourceList:
    """
    Normalize ELB listeners.

    Converts listeners from boto3 format to Ansible format, applying action and rule sorting.
    DefaultActions are sorted by Order and Type for consistent output.

    Parameters:
        listeners (list): List of listeners from boto3

    Returns:
        List of listeners in Ansible format with sorted default actions and sorted rules
    """
    if not listeners:
        return listeners

    # Transform each listener, applying nested transforms to DefaultActions and Rules
    # (force_tags=False to prevent automatic tags field)
    transforms = {
        "DefaultActions": _normalize_listener_actions,
        "Rules": _normalize_listener_rules,
    }
    return [
        boto3_resource_to_ansible_dict(listener, nested_transforms=transforms, force_tags=False)
        for listener in listeners
    ]


def normalize_application_load_balancer(alb: BotoResource) -> AnsibleAWSResource:
    """
    Normalize an Application Load Balancer from boto3 format to Ansible format.

    Handles conversion of the ALB and its nested listeners and rules. Listener rules
    are sorted by priority with the "default" rule appearing last.

    If the ALB dict contains an "Attributes" key (from describe_load_balancer_attributes),
    the attributes will be converted and merged as flat fields into the result.

    Parameters:
        alb (dict): Application Load Balancer dict in boto3 format

    Returns:
        Normalized ALB dict in Ansible format
    """
    if not alb:
        return alb

    # Make a copy to avoid modifying the original
    alb_copy = alb.copy()

    # Extract and normalize attributes separately (they get flattened into the result)
    attributes = None
    if "Attributes" in alb_copy:
        attributes = _normalize_attributes(alb_copy.pop("Attributes"))

    # Transform the ALB, applying nested transforms to Listeners
    # force_tags=False because tags require a separate API call and should be added by caller
    transforms = {"Listeners": _normalize_listeners}
    result = boto3_resource_to_ansible_dict(alb_copy, nested_transforms=transforms, force_tags=False)

    # Merge normalized attributes as flat fields
    if attributes:
        result.update(attributes)

    return result
